@@ -11,20 +11,17 @@
 
 import os
 import shutil
-import stat
 import sys
 import tarfile
 import hashlib
+import platform
+import zipfile
 from subprocess import Popen, PIPE
+from tempfile import TemporaryDirectory
+from unittest import mock
+from urllib.request import urlopen
 
-try:
-    # py2
-    from urllib2 import urlopen
-except ImportError:
-    # py3
-    from urllib.request import urlopen
-
-from .msg import fatal, debug, info, warn
+from .msg import fatal, info, warn
 
 pjoin = os.path.join
 
@@ -32,20 +29,39 @@ pjoin = os.path.join
 # Constants
 # -----------------------------------------------------------------------------
 
-bundled_version = (4, 3, 3)
+bundled_version = (4, 3, 4)
 vs = '%i.%i.%i' % bundled_version
+x, y, z = bundled_version
 libzmq = "zeromq-%s.tar.gz" % vs
-libzmq_url = "https://github.com/zeromq/libzmq/releases/download/v{vs}/{libzmq}".format(
-    vs=vs,
-    libzmq=libzmq,
-)
+download_url = f"https://github.com/zeromq/libzmq/releases/download/v{vs}"
+
+libzmq_url = f"{download_url}/{libzmq}"
 libzmq_checksum = (
-    "sha256:9d9285db37ae942ed0780c016da87060497877af45094ff9e1a1ca736e3875a2"
+    "sha256:c593001a89f5a85dd2ddf564805deb860e02471171b3f204944857336295c3e5"
 )
 
 HERE = os.path.dirname(__file__)
 ROOT = os.path.dirname(HERE)
 
+vcversion = 141
+
+if platform.architecture()[0] == '64bit':
+    msarch = '-x64'
+else:
+    msarch = ''
+
+libzmq_dll = f"libzmq-v{vcversion}{msarch}-{x}_{y}_{z}.zip"
+libzmq_dll_url = f"{download_url}/{libzmq_dll}"
+
+libzmq_dll_checksums = {
+    "libzmq-v140-4_3_4.zip": "sha256:05b7c42fe8d5feb2795d32f71f7d900083530ee6fdd15575bfc8d1b3fb8643f7",
+    "libzmq-v140-x64-4_3_4.zip": "sha256:d5d75bd502d7935af3cf80734f81069be78420331c54814d0aab6d64adf450ae",
+    "libzmq-v141-4_3_4.zip": "sha256:acfc997f036018b8dc8ab5b3a1d1444bba6ba5621e91c756d07cd9447db19672",
+    "libzmq-v141-x64-4_3_4.zip": "sha256:4bb29d6fed20bd175a82317676c7e940356cd358b624efae8569c7ee11c45636",
+}
+
+libzmq_dll_checksum = libzmq_dll_checksums.get(libzmq_dll)
+
 # -----------------------------------------------------------------------------
 # Utilities
 # -----------------------------------------------------------------------------
@@ -76,12 +92,16 @@ def checksum_file(scheme, path):
 def fetch_archive(savedir, url, fname, checksum, force=False):
     """download an archive to a specific location"""
     dest = pjoin(savedir, fname)
-    scheme, digest_ref = checksum.split(':')
+    if checksum:
+        scheme, digest_ref = checksum.split(':')
+    else:
+        scheme = "sha256"
+        digest_ref = None
 
     if os.path.exists(dest) and not force:
         info("already have %s" % dest)
         digest = checksum_file(scheme, fname)
-        if digest == digest_ref:
+        if digest == digest_ref or not digest_ref:
             return dest
         else:
             warn("but checksum %s != %s, redownloading." % (digest, digest_ref))
@@ -94,11 +114,13 @@ def fetch_archive(savedir, url, fname, checksum, force=False):
     with open(dest, 'wb') as f:
         f.write(req.read())
     digest = checksum_file(scheme, dest)
-    if digest != digest_ref:
+    if digest_ref and digest != digest_ref:
         fatal(
             "%s %s mismatch:\nExpected: %s\nActual  : %s"
             % (dest, scheme, digest_ref, digest)
         )
+    elif not digest_ref:
+        warn(f"No digest to check, got: {scheme}:{digest}")
     return dest
 
 
@@ -169,44 +191,64 @@ def stage_platform_hpp(zmqroot):
     shutil.copy(pjoin(platform_dir, 'platform.hpp'), platform_hpp)
 
 
-def copy_and_patch_libzmq(ZMQ, libzmq):
-    """copy libzmq into source dir, and patch it if necessary.
+def fetch_libzmq_dll(savedir):
+    """Download binary release of libzmq for windows
 
-    This command is necessary prior to running a bdist on Linux or OS X.
+    vcversion specifies the MSVC runtime version to use
     """
-    if sys.platform.startswith('win'):
-        return
-    # copy libzmq into zmq for bdist
-    local = localpath('zmq', libzmq)
-    if not ZMQ and not os.path.exists(local):
-        fatal(
-            "Please specify zmq prefix via `setup.py configure --zmq=/path/to/zmq` "
-            "or copy libzmq into zmq/ manually prior to running bdist."
-        )
-    try:
-        # resolve real file through symlinks
-        lib = os.path.realpath(pjoin(ZMQ, 'lib', libzmq))
-        print("copying %s -> %s" % (lib, local))
-        shutil.copy(lib, local)
-    except Exception:
-        if not os.path.exists(local):
-            fatal(
-                "Could not copy libzmq into zmq/, which is necessary for bdist. "
-                "Please specify zmq prefix via `setup.py configure --zmq=/path/to/zmq` "
-                "or copy libzmq into zmq/ manually."
-            )
 
-    if sys.platform == 'darwin':
-        # chmod u+w on the lib,
-        # which can be user-read-only for some reason
-        mode = os.stat(local).st_mode
-        os.chmod(local, mode | stat.S_IWUSR)
-        # patch install_name on darwin, instead of using rpath
-        cmd = ['install_name_tool', '-id', '@loader_path/../%s' % libzmq, local]
-        try:
-            p = Popen(cmd, stdout=PIPE, stderr=PIPE)
-        except OSError:
-            fatal("install_name_tool not found, cannot patch libzmq for bundling.")
-        out, err = p.communicate()
-        if p.returncode:
-            fatal("Could not patch bundled libzmq install_name: %s" % err, p.returncode)
+    dest = pjoin(savedir, 'zmq.h')
+    if os.path.exists(dest):
+        info("already have %s" % dest)
+        return
+    path = fetch_archive(
+        savedir, libzmq_dll_url, fname=libzmq_dll, checksum=libzmq_dll_checksum
+    )
+    archive = zipfile.ZipFile(path)
+    to_extract = []
+    for name in archive.namelist():
+        if not name.endswith(".exe"):
+            to_extract.append(name)
+    archive.extractall(savedir, members=to_extract)
+    archive.close()
+
+
+def check_checksums():
+    """Check all the checksums!"""
+    _failed = False
+
+    def less_fatal(msg):
+        """Mock fatal log message to set failed flag instead of exiting
+
+        So we can see multiple failures in one run,
+        e.g. when updating the bundled libzmq version.
+        """
+        warn(msg)
+        nonlocal _failed
+        _failed = True
+
+    with TemporaryDirectory() as savedir, mock.patch.dict(globals(), fatal=less_fatal):
+        fetch_archive(
+            savedir,
+            libzmq_url,
+            fname=libzmq,
+            checksum=libzmq_checksum,
+        )
+        for dll, checksum in libzmq_dll_checksums.items():
+            fetch_archive(
+                savedir,
+                f"{download_url}/{dll}",
+                fname=dll,
+                checksum=checksum,
+            )
+    if not _failed:
+        print("All ok!")
+    return _failed
+
+
+if __name__ == "__main__":
+    # allow python -m buildutils.bundle to get bundled version
+    if len(sys.argv) > 1 and sys.argv[1] == "checksums":
+        sys.exit(check_checksums())
+    else:
+        print(vs)
